from typing import TYPE_CHECKING, Optional
from cosmic_text_py.lib import Color

if TYPE_CHECKING:
    from PIL import Image


class Canvas:
    def __new__(self, width: int, height: int, color: Color) -> Canvas:
        """Create a new canvas.

        Args:
            width (int): The width of the canvas.
            height (int): The height of the canvas.
            color (Color): The color of the canvas.

        Returns:
            Canvas: The canvas.
        """

    def save(self, path: str) -> None:
        """Save the canvas to a file.

        Args:
            path (str): The path to save the file to.
        """

    def to_bytes(self) -> tuple[tuple[int, int], bytes]:
        """Get the canvas as bytes.

        Returns:
            tuple[tuple[int, int], bytes]: The canvas dimensions and bytes.
        """

    def to_image(self) -> "Image.Image":
        """Get the canvas as a PIL image.

        Returns:
            The canvas as an image.
        """

    def to_buffer(self) -> list[int]:
        """Get the canvas as a buffer.

        Returns:
            list[int]: The canvas as a buffer.
        """

    @staticmethod
    def from_image(image: "Image.Image") -> Canvas:
        """Create a canvas from an image.

        Args:
            image (Image.Image): The image.

        Returns:
            Canvas: The canvas.
        """


class Font:
    def __new__(self, size: float, path: str, fallbacks: list[str]=None) -> Font:
        """Create a new font.

        Args:
            size (float): The font size
            path (str): The path to the font.
            fallbacks (list[str], optional): The fallback fonts. Defaults to None.

        Returns:
            Font: The font.
        """


def draw_text(
    canvas: Canvas,
    text: str,
    x: float,
    y: float,
    width: float,
    height: float,
    font: Font,
    font_color: Color
) -> None:
    ...


def draw_text_advance(
    canvas: Canvas,
    text: tuple[tuple[str, Color], ...],
    x: float,
    y: float,
    width: float,
    height: float,
    font: Font,
) -> None:
    ...



from typing import Self

from cosmic_text_py.cosmic_text_py import *

try:
    from PIL import Image
except:
    pass


class Color(tuple):
    def __new__(cls, r: int, g: int, b: int, a: int = 255) -> Self:
        """Create a new color.

        Args:
            r (int): The red value of the color.
            g (int): The green value of the color.
            b (int): The blue value of the color.
            a (int, optional): The alpha value of the color. Defaults to 255.
        """
        return super().__new__(cls, (r, g, b, a))

    @staticmethod
    def from_hex(hex: str) -> Self:
        """Create a color from a hex string.

        Args:
            hex (str): The hex string.

        Returns:
            Color: The color.
        """
        return Color(*bytes.fromhex(hex))

    def __repr__(self) -> str:
        return f"Color({self.r}, {self.g}, {self.b}, {self.a})"

    @property
    def r(self) -> int:
        """The red value of the color."""
        return self[0]

    @property
    def g(self) -> int:
        """The green value of the color."""
        return self[1]

    @property
    def b(self) -> int:
        """The blue value of the color."""
        return self[2]

    @property
    def a(self) -> int:
        """The alpha value of the color."""
        return self[3]


class Writer:
    def __init__(self, image: "Image.Image") -> None:
        """Create a new draw object.

        Args:
            image (Image.Image): The image to draw on.
        """
        self.image = image
        self.__canvas = None

    def __enter__(self) -> Self:
        assert self.image.mode == "RGBA", "The image must be in RGBA mode."
        self.__canvas = Canvas.from_image(self.image)
        return self

    def __exit__(self, exc_type, exc_value, traceback) -> None:
        self.image.frombytes(self.__canvas.to_bytes()[1])
        self.__canvas = None

    @property
    def _canvas(self) -> Canvas:
        assert self.__canvas is not None, "You must use the Writer as a context manager."
        return self.__canvas

    def draw_text(
        self,
        text: str,
        x: float,
        y: float,
        width: float,
        height: float,
        font: Font,
        font_color: Color
    ) -> None:
        draw_text(self._canvas, text, x, y, width, height, font, font_color)
